package com.jkcredit.illegal.info.task;

import com.jkcredit.illegal.info.constant.CommonConstant;
import com.jkcredit.illegal.info.model.SendDingMsgResponseObject;
import com.jkcredit.illegal.info.util.DingRobotUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.text.DecimalFormat;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Component
@Slf4j
public class ExceptionListenerTask {
    @Autowired
    StringRedisTemplate stringRedisTemplate;


    @Scheduled(cron = "0 0/15 * * * ?", zone = "Asia/Shanghai")
    public void personIllegalInfoTask() {
        DecimalFormat decimalFormat = new DecimalFormat("0.00%");
        double timeOutCount = 0;
        double queryFailedCount = 0;
        double totalCount = 0;
        String timeOut = stringRedisTemplate.boundValueOps(CommonConstant.PERSON_ILLEGAL_INFO_TIME_OUT).get();
        String queryFailed = stringRedisTemplate.boundValueOps(CommonConstant.PERSON_ILLEGAL_INFO_QUERY_FAILED).get();
        String total = stringRedisTemplate.boundValueOps(CommonConstant.PERSON_ILLEGAL_INFO_COUNT).get();
        if (timeOut != null) {
            timeOutCount = Double.parseDouble(timeOut);
        }
        if (queryFailed != null) {
            queryFailedCount = Double.parseDouble(queryFailed);
        }
        if (total != null) {
            totalCount = Double.parseDouble(total);
        }

        StringBuilder errorMessage = new StringBuilder("社会风险预警-人员核验接口失败，失败原因：");
        if (timeOutCount > 10) {
            errorMessage.append("数据源超时, 超时比例 ").append(decimalFormat.format(timeOutCount / totalCount));
            SendDingMsgResponseObject sendDingMsgResponseObject = DingRobotUtil.sendDingRobotMsg(errorMessage.toString());
            if (sendDingMsgResponseObject.getCode() != 1) {
                log.info("personIllegalInfoTask, 发送钉钉异常：社会风险预警-人员核验接口失败-数据源超时");
            }
        }

        if (queryFailedCount > 10) {
            errorMessage.append("数据源返回结果异常，异常比例 ").append(decimalFormat.format(queryFailedCount / totalCount));
            SendDingMsgResponseObject sendDingMsgResponseObject = DingRobotUtil.sendDingRobotMsg(errorMessage.toString());
            if (sendDingMsgResponseObject.getCode() != 1) {
                log.info("personIllegalInfoTask, 发送钉钉异常：社会风险预警-人员核验接口失败-数据源返回结果异常");
            }
        }
        stringRedisTemplate.delete(CommonConstant.PERSON_ILLEGAL_INFO_TIME_OUT);
        stringRedisTemplate.delete(CommonConstant.PERSON_ILLEGAL_INFO_QUERY_FAILED);
        stringRedisTemplate.delete(CommonConstant.PERSON_ILLEGAL_INFO_COUNT);
    }

    @Scheduled(cron = "0 0/15 * * * ?", zone = "Asia/Shanghai")
    public void vehicleIllegalInfoTask() {
        DecimalFormat decimalFormat = new DecimalFormat("0.00%");
        double timeOutCount = 0;
        double queryFailedCount = 0;
        double totalCount = 0;
        String timeOut = stringRedisTemplate.boundValueOps(CommonConstant.VEHICLE_ILLEGAL_INFO_TIME_OUT).get();
        String queryFailed = stringRedisTemplate.boundValueOps(CommonConstant.VEHICLE_ILLEGAL_INFO_QUERY_FAILED).get();
        String total = stringRedisTemplate.boundValueOps(CommonConstant.VEHICLE_ILLEGAL_INFO_COUNT).get();
        if (timeOut != null) {
            timeOutCount = Integer.parseInt(timeOut);
        }
        if (queryFailed != null) {
            queryFailedCount = Integer.parseInt(queryFailed);
        }
        if (total != null) {
            totalCount = Integer.parseInt(total);
        }

        StringBuilder errorMessage = new StringBuilder("社会风险预警-车辆核验接口失败，失败原因：");
        if (timeOutCount > 10) {
            errorMessage.append("数据源超时，超时比例 ").append(decimalFormat.format(timeOutCount / totalCount));
            SendDingMsgResponseObject sendDingMsgResponseObject = DingRobotUtil.sendDingRobotMsg(errorMessage.toString());
            if (sendDingMsgResponseObject.getCode() != 1) {
                log.info("vehicleIllegalInfoTask, 发送钉钉异常：社会风险预警-车辆核验接口失败-数据源超时");
            }
        }

        if (queryFailedCount > 10) {
            errorMessage.append("数据源返回结果异常，异常比例 ").append(decimalFormat.format(queryFailedCount / totalCount));
            SendDingMsgResponseObject sendDingMsgResponseObject = DingRobotUtil.sendDingRobotMsg(errorMessage.toString());
            if (sendDingMsgResponseObject.getCode() != 1) {
                log.info("vehicleIllegalInfoTask, 发送钉钉异常：社会风险预警-车辆核验接口失败-数据源返回结果异常");
            }
        }
        stringRedisTemplate.delete(CommonConstant.VEHICLE_ILLEGAL_INFO_TIME_OUT);
        stringRedisTemplate.delete(CommonConstant.VEHICLE_ILLEGAL_INFO_QUERY_FAILED);
        stringRedisTemplate.delete(CommonConstant.VEHICLE_ILLEGAL_INFO_COUNT);
    }
}
